import asyncio
import httpx
import feedparser
import pandas as pd

from .config import Config
from .source import Source, SourceGroup
from .backend import Loader
from .utils import eprint

from dataclasses import dataclass
from typing import List, Tuple
from urllib.parse import urlparse
from pathlib import Path


RESPONSE_OK_STATUS = 200

class PressException(Exception):
    def __init__(self, message, exc):
        self.message = message
        self.exc = exc
        super().__init__(self.message, exc)


def is_path(s: str):
    return urlparse(s).scheme == ""

FetchedSource = Tuple[Source, pd.DataFrame]
FetchedSourceGroup = Tuple[SourceGroup, List[FetchedSource]]

@dataclass
class Press:
    config: Config

    async def fetch_source(self, client: httpx.AsyncClient, source: Source) -> FetchedSource:
        try:
            # Fetch feed
            if is_path(source.url):
                path = Path(source.url).expanduser()
                with open(path, 'r') as f:
                    feed = f.read()
            else:
                response = await client.get(source.url, follow_redirects=True)
                response.raise_for_status()
                feed = response.text

            # Parse feed
            data = feedparser.parse(feed)

            # Convert to data frame
            df = pd.DataFrame(data["entries"])

            eprint(f"Fetched '{source}'")

            return (source, df)
        except Exception as e:

            eprint(f"FAILED to fetch source '{source}'. Exception = '{e}'")

            raise PressException(f"Failed to fetch source '{source}'", e)

    async def fetch_source_group(self, client: httpx.AsyncClient, source_group: SourceGroup) -> FetchedSourceGroup:
        tasks = [
            self.fetch_source(client, source)
            for source in source_group.sources
        ]
        sources = await asyncio.gather(*tasks, return_exceptions=True)
        sources = [source for source in sources if not isinstance(source, Exception)]
        return (source_group, sources)

    async def fetch_source_groups(self) -> List[FetchedSourceGroup]:
        async with httpx.AsyncClient() as client:
            tasks = [
                self.fetch_source_group(client, source_group)
                for source_group in self.config.source_groups
            ]
            data = await asyncio.gather(*tasks)
        return data

    async def run(self, backend_key: str, output_dir: Path) -> Path:
        loader = Loader(self.config.settings.backends_dir)
        backend = loader.load(backend_key)()
        data = await self.fetch_source_groups()
        output_dir = output_dir / Path(backend_key)
        if not output_dir.exists():
            output_dir.mkdir(parents=True)
        return await backend.run(data, output_dir)



from typing import List
from pathlib import Path
from datetime import datetime, timedelta, timezone
from dateutil.parser import parse as date_parse

from guten.backend import Backend
from guten.press import FetchedSourceGroup


METADATA_FILE = ".metadata"


def underline(s: str, c = '-') -> str:
    return f"{s}\n{c*len(s)}\n\n"


class TextBackend(Backend):
    async def run(self, groups: List[FetchedSourceGroup], output_dir: Path) -> Path:
        today = datetime.now(timezone.utc)
        output_file = output_dir / f"{today.year}-{today.month}-{today.day}.txt"
        metadata_file = output_dir / METADATA_FILE

        previous_run_date = today - timedelta(days=5)
        if metadata_file.exists():
            with open(metadata_file, "r") as f:
                data = f.read()
            temp = date_parse(data)
            if temp.date() != today.date():
                previous_run_date = temp
            else:
                previous_run_date = today - timedelta(days=1)

        with open(output_file, "w") as f:
            localdate = datetime.now()
            f.write(underline(f"Feed - {locadate.year}/{localdate.month}/{localdate.day}", c="="))
            f.write(f"Generated at: {today.isoformat()}\n\n")

            def process_source(item):
                source, df = item
                if df.empty:
                    return source, []
                data = df
                if "published" in df:
                    df["date"] = df["published"].apply(lambda x: date_parse(x))
                    data = df[df["date"] > previous_run_date]
                data = data[["title", "link"]]
                data = data.apply(lambda x: f"- [{x['title']}]({x['link']})", axis=1)
                data = list(data)
                return source, data

            for (group, sources) in groups:
                sources = [process_source(source) for source in sources]
                sources = [(source, data) for (source, data) in sources if len(data) > 0]
                if len(sources) == 0:
                    continue

                f.write(underline(group.name.title(), c="~"))
                for source, data in sources:
                    f.write(underline(source.name, c="-"))
                    f.write("\n".join(data))
                f.write("\n\n")

        with open(metadata_file, "w") as f:
            f.write(today.isoformat())

        return output_file

__backend__ = TextBackend



from typing import List
from pathlib import Path
from datetime import datetime, timedelta, timezone
from dateutil.parser import parse as date_parse

from guten.backend import Backend
from guten.press import FetchedSourceGroup


METADATA_FILE = ".metadata"

def begin_document():
    return "<html>"

def end_document():
    return "</html>"

def preamble():
    return """
    <head>
        <style>
            html {
                margin: 20px 60px;
            }

            body {
                font-size: 16px;
                font-family: sans-serif;
            }

            h1 {
                border: 10px double black;
                text-align: center;
                padding: 10px;
            }

            details {
                margin: 5px 0px;
                padding: 5px 15px;
                border-radius: 4px;
            }

            details:hover {
                /* background: #ffffd0; */
            }

            a {
                margin-left: 10px;
            }
        </style>
    </head>
    """

def begin_body():
    return "<body>"

def end_body():
    return "</body>"

def h1(s):
    return f"<h1>{s}</h1>"

def h2(s):
    return f"<h2>{s}</h2>"

def h3(s):
    return f"<h3>{s}</h3>"

def small(s):
    return f"<small>{s}</small>"

def begin_list():
    return "<ul>"

def end_list():
    return "</ul>"

def source_item(data):
    return f"""
        <details>
            <summary><a href='{data['link']}'>{data['title']}</a></summary>
            {data['summary']}
        </details>
    """

class HTMLBackend(Backend):
    async def run(self, groups: List[FetchedSourceGroup], output_dir: Path) -> Path:
        today = datetime.now(timezone.utc)
        output_file = output_dir / f"{today.year}-{today.month}-{today.day}.html"
        metadata_file = output_dir / METADATA_FILE

        previous_run_date = today - timedelta(days=5)
        if metadata_file.exists():
            with open(metadata_file, "r") as f:
                data = f.read()
            temp = date_parse(data)
            if temp.date() != today.date():
                previous_run_date = temp
            else:
                previous_run_date = today - timedelta(days=1)

        with open(output_file, "w") as f:
            f.write(begin_document())
            f.write(preamble())

            f.write(begin_body())

            localdate = datetime.now()
            f.write(h1(f"{localdate.year}/{localdate.month}/{localdate.day}"))

            def process_source(item):
                source, df = item
                if df.empty:
                    return source, []
                data = df
                if "published" in df:
                    df["date"] = df["published"].apply(lambda x: date_parse(x))
                    data = df[df["date"] > previous_run_date]
                data = data[["title", "link", "summary"]]
                data = data.apply(lambda x: source_item(x), axis=1)
                data = list(data)
                return source, data


            for (group, sources) in groups:
                sources = [process_source(source) for source in sources]
                sources = [(source, data) for (source, data) in sources if len(data) > 0]
                if len(sources) == 0:
                    continue

                f.write(h2(group.name.title()))
                for source, data in sources:
                    f.write(h3(source.name))
                    # f.write(begin_list())
                    f.write("".join(data))
                    # f.write(end_list())

            f.write(small(f"Generated at: {today.isoformat()}"))

            f.write(end_body())
            f.write(end_document())

        with open(metadata_file, "w") as f:
            f.write(today.isoformat())

        return output_file



__backend__ = HTMLBackend



import importlib
import os.path
import pandas as pd

from dataclasses import dataclass
from pathlib import Path
from abc import ABC, abstractmethod
from typing import List

from .utils import prepend_path
from .source import SourceGroup


class LoaderException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class Backend(ABC):
    @abstractmethod
    async def run(self, groups: List[SourceGroup], output_dir: Path) -> Path:
        pass


class Loader:
    backends_dir: Path

    def __init__(self, backends_dir: Path):
        if not backends_dir.exists():
            raise LoaderException(f"Backends dir '{backends_dir}' does not exist")
        self.backends_dir = os.path.realpath(backends_dir)

    def load(self, key: str):
        with prepend_path(self.backends_dir):
            try:
                module = importlib.import_module(f"backends.{key}")
            except:
                module = importlib.import_module(f"guten.backends.{key}")
        if not hasattr(module, '__backend__'):
            raise LoaderException(f'Module {module} is missing a __backend__ attr')

        return getattr(module, '__backend__')



import sys
import argparse
import toml
import asyncio

from pathlib import Path

from .config import Config, DEFAULT_CONFIG_PATH
from .press import Press


async def main():
    parser = argparse.ArgumentParser(description="guten - compile rss/atom feeds into a combined feed")
    parser.add_argument("--config-file", type=str, help="Path to config file", default=str(DEFAULT_CONFIG_PATH))
    parser.add_argument("--output-dir", type=str, help="Path to output directory", default=str(Path(".")))
    parser.add_argument("--backend", type=str, help="Backend to use", default="html")

    args = parser.parse_args()

    config = Config.from_file(args.config_file)
    press = Press(config)
    output_file = await press.run(
        backend_key=args.backend,
        output_dir=args.output_dir
    )
    print(output_file)

if __name__ == "__main__":
    asyncio.run(main())



import toml

from pathlib import Path
from typing import List, Dict, Any
from dataclasses import dataclass

from .source import Source, SourceGroup


SOURCES_KEY = "sources"
SETTINGS_KEY = "settings"

NOGROUP_NAME = "<nogroup>"


class ConfigParseException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


def default_config_dir():
    path = Path("~/.config/guten").expanduser()
    if not path.exists():
        path.mkdir(parents=True)
    return path


DEFAULT_CONFIG_DIR = default_config_dir()
DEFAULT_CONFIG_FILE_NAME = "config.toml"
DEFAULT_CONFIG_PATH = DEFAULT_CONFIG_DIR / DEFAULT_CONFIG_FILE_NAME
DEFAULT_BACKENDS_DIR = DEFAULT_CONFIG_DIR


@dataclass
class Settings:
    backends_dir: Path


def default_settings():
    return Settings(
        backends_dir=Path(DEFAULT_BACKENDS_DIR)
    )


@dataclass
class Config:
    settings: Settings
    source_groups: List[SourceGroup]


    @classmethod
    def from_dict(cls, config_dict: Dict[str, Any]):
        # Parse settings
        if SETTINGS_KEY not in config_dict:
            settings = default_settings()
        else:
            # TODO more robust implementation
            settings_dict = config_dict["settings"]
            settings = Settings(
                backends_dir=Path(settings_dict["backends_dir"]).expanduser()
            )

        # Parse source groups
        if SOURCES_KEY not in config_dict:
            raise ConfigParseException(f"Missing '{SOURCES_KEY}' key in config")

        sources_dict = config_dict["sources"]

        nogroup = SourceGroup(NOGROUP_NAME, [])

        # invariant: nogroup is always at the head
        source_groups = [nogroup]

        for (groupname, sources) in sources_dict.items():
            if isinstance(sources, str):
                name = groupname
                url = sources
                nogroup.sources.append(Source(name, url))
            elif isinstance(sources, dict):
                group = SourceGroup(groupname, [])
                for (name, url) in sources.items():
                    if not isinstance(url, str):
                        raise ConfigParseException("Multi-level nesting of source groups is not permitted", None)
                    group.sources.append(Source(name, url))
                source_groups.append(group)
            else:
                raise ConfigParseException(f"Failed to parse '{SOURCES_KEY}' config", None)

        return cls(source_groups=source_groups, settings=settings)


    @classmethod
    def from_str(cls, config_str: str):
        try:
            config_dict = toml.loads(config_str)
            return Config.from_dict(config_dict)
        except ConfigParseException as e:
            raise e
        except Exception as e:
            raise ConfigParseException("Failed to parse config string. It should be a valid TOML string", e)


    @classmethod
    def from_file(cls, path: Path):
        try:
            config_dict = toml.load(path)
            return Config.from_dict(config_dict)
        except ConfigParseException as e:
            raise e
        except Exception as e:
            raise ConfigParseException(f"Failed to parse config file '{str(path)}'. It should be a valid TOML file", e)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['guten', 'guten.backends']

package_data = \
{'': ['*']}

install_requires = \
['feedparser>=6.0.10,<7.0.0',
 'httpx>=0.23.1,<0.24.0',
 'pandas>=1.5.2,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'pytz>=2022.7,<2023.0',
 'toml==0.10.2']

setup_kwargs = {
    'name': 'guten',
    'version': '0.1.7',
    'description': '',
    'long_description': '# guten\n\nCompile RSS/Atom feeds into a combined feed.\n',
    'author': 'Elton',
    'author_email': 'eltonp3103@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import sys

class prepend_path:
    def __init__(self, path):
        self.path = path

    def __enter__(self):
        sys.path.insert(0, self.path)

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            sys.path.remove(self.path)
        except:
            pass

def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)
